function [aut,ee]=autoval_rev1(beta,d,Fr,omega,alp,N)
% The present function computes the spectrum of the eigenvalue problem
% (3.10) reported in the companion paper Camporeale, Cannamela, Canuto, Manes, 
% submitted to JFM. For further information: carlo.camporeale@polito.it
% With reference to the notation of the same paper:
% Input: 
% \beta = spanwise wavenumber of secondary cells (2\pi/L_z)
% d =  mean grain size
% Fr = Froude number
% \omega  (see eq.2.4)
% \alpha = longitudinal wavenumber of LSM (2\pi/L_x)
% N = truncation number (>20)

% Output:
% aut= the least stable eignevalue (i.e., the one with largest real part)
% ee= the whole set of the eigenvalues

%Physical constants
kappa = 0.41; %von-Karman costant
y0 = (2.2*d)/30; % roughness thickness
Cf0=kappa^2/(-1+y0-log(y0))^2; %Friction factor
uas=sqrt(Cf0); %friction velocity
CfD=(-2*kappa^2*(1 - y0))/(-1+y0-log(y0))^3; % partial derivative d(Cf0)/dh

%Integral Terms
I0=(kappa*(5 + y0^2*(-9 + 4*y0) + 6*log(y0)))/(36*(1 - y0 + log(y0)));
I1=kappa/6;
%I2=(cosint(beta) - (beta*cosint(beta*y0) + beta*cos(beta)*log(y0) + 2*sin(beta) - 2*sin(beta*y0))/beta)/(-1 + y0 - log(y0));                            % Vecchia versione errata
I2=(beta.*(cosint(beta)-cosint(y0.*beta)+log(y0))+(2.*sec(beta).*sin(beta./2).^2).*(sin(beta)-sin(y0.*beta)))./(beta.*(-1+y0-log(y0)));
I4=(Cf0*(2 - 2*y0 + log(y0)*(2 + log(y0))))/kappa^2;

%Coefficients sigma_i 
den=beta^2*I0 + 2*uas;
s0=2*alp^2*I0 + (1i*alp*(2*I4-1))/uas + 2*uas;
s1=(-CfD +1i*alp*(1-Fr^(-2)-I4) + uas^2)/uas;
s2=(alp*beta*I2*(-1i + (2*1i)*I4 + 2*alp*I0*uas))/(uas^2*(den));
s3=(beta*I2*(2*CfD + (2*1i)*alp*(-1 + I4) + beta^2*I0*uas - 2*uas^2))/...
 (uas^2*(den));
s4=alp^2*I1 + (1i*alp)/uas + uas;
s5=((-2*1i)*alp*I2)/(den);
s6=((-1i)*alp*beta*I0*I2)/(den*uas);
s7=(beta*I2)/(uas*(den));
s8=(-2*beta*I1*I2)/(uas*den);
s9=I2*((1/uas)-(4/den));

%Construction of Matrices \hat{A}_n, \hat{B}_{-1,n}, \hat{B}_{1,n}
    
    L=zeros(3*(N+1),3*(N+1)); %Initizalization

    %First row-block
    i=1;n=-N/2;
    Ak=[-I0*n^2*beta^2-s0, -n*I1*alp*beta,s1;...
        -n*I0*alp*beta, -2*I1*n^2*beta^2-s4, -1i*beta*n/(uas*Fr^2);...
        -1i*alp/uas, -1i*n*beta/uas,-1i*alp/uas];
    Bkm1=omega*M1(n+1,beta,s2,uas,alp,s3,s4,s5,s6,s7,s8,s9);             
    L(i:i+2,i:i+5)=[Ak Bkm1];

    i=4;

    %core
    for n=-N/2+1:1:N/2-1    
    Ak=[-I0*n^2*beta^2-s0, -n*I1*alp*beta,s1;...
        -n*I0*alp*beta, -2*I1*n^2*beta^2-s4, -1i*beta*n/(uas*Fr^2);...
        -1i*alp/uas, -1i*n*beta/uas,-1i*alp/uas];
    Bkm1=omega*M1(n+1,beta,s2,uas,alp,s3,s4,s5,s6,s7,s8,s9); 
    Bkp1=omega*P1(n-1,beta,s2,uas,alp,s3,s4,s5,s6,s7,s8,s9); 
    L(i:i+2,i-3:i+5)=[Bkp1 Ak Bkm1];
    i=i+3;
    end

    %Last row-block
    n=N/2;
    Ak=[-I0*n^2*beta^2-s0, -n*I1*alp*beta,s1;...
        -n*I0*alp*beta, -2*I1*n^2*beta^2-s4, -1i*beta*n/(uas*Fr^2);...
        -1i*alp/uas, -1i*n*beta/uas,-1i*alp/uas];
    Bkp1=omega*P1(n-1,beta,s2,uas,alp,s3,s4,s5,s6,s7,s8,s9);
    L(i:i+2,i-3:i+2)=[Bkp1 Ak];

% Eigenvalue Problem
[~,D]=eig(uas*L);
ee=diag(D);

% Ordering
[~,ind]=max(real(diag(D)));
aut=D(ind,ind);

function Bm1=M1(k,beta,s2,uas,alp,s3,s4,s5,s6,s7,s8,s9)
Bm1=[s2/2 + 1i*k^2*beta^2*s6/(2*alp) - k*beta*s9/2,...
        -1i*beta*s7/(2*uas) + alp*beta*s8/4 - k*alp*beta*s8/4, s3/2 - k*beta*s9/2;...
         (1i/2)*s5 + (1i/2)*beta*k*s6, s7*s4/2 - (beta^2*k^2*s8)/2 + k*beta^2*s8/2,...
         -alp*s9/2;...
         0 0 +1i*alp*s7/(2*uas)];       
end

function Bp1=P1(k,beta,s2,uas,alp,s3,s4,s5,s6,s7,s8,s9)
Bp1=[s2/2 + 1i*k^2*beta^2*s6/(2*alp) + k*beta*s9/2,...
        1i*beta*s7/(2*uas) - alp*beta*s8/4 - k*alp*beta*s8/4, s3/2 + k*beta*s9/2;...
         -(1i/2)*s5 + (1i/2)*beta*k*s6, s7*s4/2 - (beta^2*k^2*s8)/2 - k*beta^2*s8/2,...
         alp*s9/2;...
         0 0 +1i*alp*s7/(2*uas)];    
end

end     
